PROGRAM Fisher_Distance
  USE cosmo
  USE growth
  USE linearpk
  USE angular_distance
  ! A sample program for computing the errors on ln(Da) and ln(H), 
  ! from the information of Baryon Acoustic Oscillations only.
  ! Ref: Seo & Eisenstein, ApJ, 665, 14 (2007)
  ! September 17, 2008: E.Komatsu
  ! Modified to report also on the errors on w, Omega_k, and Omega_m.
  ! November 2, 2008: E.Komatsu
  IMPLICIT none
  integer :: npara=2 ! # of parameters
  double precision, dimension(2)   :: der
  double precision, dimension(2,2) :: cov,fis,fistot
  double precision, dimension(3,3) :: fis3x3
  integer,          dimension(2)   :: work
  double precision :: sigma0=12.4d0*0.817d0/0.9d0 ! non-linearity scale in units of h Mpc^-1, rescaled by WMAP5's sigma8 value (0.817)
  double precision :: BAO_AMP=0.5817d0 ! A_0 in Seo&Eisenstein
  double precision :: kmax_ov_h,k_ov_h,k0_ov_h ! h Mpc^-1
  double precision :: linear_pk,pk,p02
  double precision :: sigma_para,sigma_perp,sigma_silk=8.38d0 ! h^-1 Mpc
  double precision :: dlnk,mu,mu2,dmu,factor,wkmu,dummy,wdamp
  double precision :: z,zin,beta,bias,g,dgdlna,sigma_z,delta_v
  double precision :: zmin,zmax,area,Vsurvey,Vtot,ngal,number,Ntot
  character(len=128) :: filename
  integer :: n,i,j,ibin,nbins,ifile
  external linear_pk,dgdlna,g
  ! Specify three cosmological parameters for computing the growth factor
  ! and the angular diameter distance. 
  ! The data type has been defined in MODULE cosmo.
  ode0=0.723d0
  om0=0.277d0
  w=-1d0
  CALL setup_growth ! tabulate the growth factor
  CALL setup_da     ! tabulate the angular diameter distance
  ! ask for survey specific parameters
  print*,'For the area, the redshift range and the number of galaxies:'
  print*,'1) Read in "number.txt"'
  print*,'2) Read in your favorite file'
  print*,'0) Enter your choice of area (in deg^2), zmin, zmax, # of galaxies in millions'
  print*,'   (A new file called "number.txt" will be created.)'
  read*,ifile
  if(ifile.eq.1)then
     filename='number.txt'
     open(2,file=filename,status='old')
  elseif(ifile.eq.2)then
     print*,'filename? (format: area in deg^2  zmin  zmax  # of galaxies in millions)'
     read*,filename
     open(2,file=filename,status='old')
  else
     filename='number.txt'
     open(2,file=filename,status='unknown')
  endif
  ! read in linear P(k)
  filename='wmap5baosn_max_likelihood_matterpower_at_z=30.dat'
  n=896    ! # of lines in the file
  zin=30d0 ! redshift of the input power spectrum
  CALL open_linearpk(filename,n)
  ! loop over redshift bins
  fistot=0d0 ! total Fisher matrix integrated over all the redshift bins
  fis3x3=0d0 ! total Fisher matrix for w, Omega_k, and Omega_m
  Ntot=0d0   ! total number of galaxies in millions
  Vtot=0d0   ! total survey volume
  nbins=0    ! initialize the number of bins
  do ibin=1,100
     if(ifile.eq.1)then
        read(2,*,end=10)area,zmin,zmax,number
     elseif(ifile.eq.2)then
        read(2,*,end=10)area,zmin,zmax,number
     else
        print*,'Enter area (in deg^2), zmin, zmax, # of galaxies (in millions)'
        print*,'in the redshift bin#=(',ibin,')'
        print*,'(Enter 0,0,0,0 when you are done.)'
        read*,area,zmin,zmax,number
        if(area.eq.0d0.and.zmin.eq.0d0.and.zmax.eq.0d0.and.number.eq.0d0)goto 10
        write(2,'(4F18.5)')area,zmin,zmax,number
     endif
     nbins=nbins+1
     print*,'Enter the values of'
     print*,'bias, kmax [h Mpc^-1], and redshift error [km/s] in '
     print'(1F6.3,1A7,1F5.3)',zmin,'< z < ',zmax
     read*,bias,kmax_ov_h,delta_v
     CALL volume(zmin,zmax,area,Vsurvey)
     z=0.5d0*(zmin+zmax)
     beta=(1d0+dgdlna(z)/g(z))/bias
     ngal=number*1d6/Vsurvey
     Ntot=Ntot+number
     Vtot=Vtot+Vsurvey
     sigma_z=(1d0+z)*(delta_v/2.998d5) &
          *(2.998d3/dsqrt(om0*(1d0+z)**3d0+ode0)) ! in units of h^-1 Mpc
     sigma_perp=sigma0*g(z)/(1d0+z) ! in units of h^-1 Mpc
     sigma_para=sigma0*g(z)/(1d0+z)*(2d0+dgdlna(z)/g(z)) ! in units of h^-1 Mpc
     ! computing the Fisher matrix...
     open(11,file=filename,status='old')
     read(11,*)k0_ov_h,dummy
     k_ov_h=k0_ov_h
     fis=0d0
     do while (k_ov_h<=kmax_ov_h)
        read(11,*)k_ov_h,dummy
        dlnk=dlog(k_ov_h)-dlog(k0_ov_h)
        factor=(k_ov_h)**3d0*dlnk/(2d0*3.1415926535d0**2d0) ! h^3 Mpc^-3
        mu=0d0
        dmu=1d-3
        do while (mu<=1d0)
           mu2=mu*mu
           p02=linear_pk(0.2d0)*(g(z)/g(zin)*(1d0+zin)/(1d0+z))**2d0 &
                *bias**2d0*(1d0+beta*mu2)**2d0
                         ! P02 is P(k) at k=0.2 h Mpc^-1 in units of h^-3 Mpc^3
           pk=linear_pk(k_ov_h)*(g(z)/g(zin)*(1d0+zin)/(1d0+z))**2d0 &
                *bias**2d0*(1d0+beta*mu2)**2d0
           wkmu=0.5d0*Vsurvey*(ngal*p02/(1d0+ngal*pk))**2d0
           wdamp=0.5d0 & ! for [cos(x)]^2 ~ 0.5
                *exp(-(k_ov_h*sigma_z)**2d0*mu2) & 
                *exp(-(k_ov_h*sigma_perp)**2d0*(1d0-mu**2d0)) & 
                *exp(-(k_ov_h*sigma_para)**2d0*mu**2d0) &
                *exp(-2d0*(k_ov_h*sigma_silk)**1.4d0)
           der(1)=(1d0-mu2) ! dlnPkdlnDa
           der(2)=(   -mu2) ! dlnPkdlnH
           do i=1,npara
              fis(i,:)=fis(i,:)+factor*wkmu*der(i)*der(:)*dmu &
                   *(8d0*3.1415926535d0**2d0*BAO_AMP**2d0)*wdamp
           enddo
           mu=mu+dmu
        enddo
        k0_ov_h=k_ov_h
     enddo
     fistot=fistot+fis
     print*,'=== redshift bin#=(',ibin,') ==='
     print'(1F6.3,1A7,1F5.3)',zmin,'< z < ',zmax
     print'(1A7,1F8.5,1A11)','Vsur =',Vsurvey*1d-9,' h^-3 Gpc^3'
     print'(1A7,1F8.5,1A9)','Ngal =',number,' millions'
     print'(1A7,1F8.5,1A17)','ngal =',ngal*1d3,' 10^-3 h^3 Mpc^-3'
     print'(1A7,1F8.5)','bias =',bias
     print'(1A7,1F8.5)','beta =',beta
     print'(1A7,1F8.5)','f    =',1d0+dgdlna(z)/g(z)
     print'(1A7,1F8.5,1A9)','sigz =',sigma_z,' h^-1 Mpc'
     print'(1A7,1F8.5,1A9)','spara=',sigma_para,' h^-1 Mpc'
     print'(1A7,1F8.5,1A9)','sperp=',sigma_perp,' h^-1 Mpc'
     print'(1A7,1F8.5,1A9)','kmax =',kmax_ov_h,' h Mpc^-1'
     CALL report_result(npara,fis)
     print*,''
     close(11)
     CALL transform_fisher(z,fis,fis3x3)
  enddo
10 close(2)
  CALL close_linearpk
  if(nbins>1)then
     print*,'=== combined ==='
     print'(1A7,1F8.5,1A11)','Vsur =',Vtot*1d-9,' h^-3 Gpc^3'
     print'(1A7,1F8.5,1A9)', 'Ngal =',Ntot,' millions'
     print'(1A7,1F8.5,1A17)','ngal =',Ntot*1d6/Vtot*1d3,' 10^-3 h^3 Mpc^-3'
     CALL report_result(npara,fistot)
  endif
  print*,''
  CALL report_result3x3(fis3x3)
  print*,''
  CALL add_cmb(fis3x3)
  CALL report_result3x3(fis3x3)
END PROGRAM Fisher_Distance
SUBROUTINE report_result(npara,fis)
  IMPLICIT none
  integer, intent(IN) :: npara
  double precision, intent(IN) :: fis(npara,npara)
  double precision, allocatable, dimension(:,:) :: cov
  double precision, allocatable, dimension(:)   :: work
  double precision :: r12,err_lnda,err_lnh,err_lnR
  integer :: i,j
  ALLOCATE(cov(2,2),work(2))
  cov=fis
  CALL DVERT(cov,2,2,work)
  r12=cov(1,2)/sqrt(cov(1,1)*cov(2,2))
  err_lnda=sqrt(cov(1,1))
  err_lnh=sqrt(cov(2,2))
  err_lnR=err_lnda*sqrt((1d0-r12**2d0) &
       /(1d0+2d0*r12*err_lnda/err_lnh+(err_lnda/err_lnh)**2d0))
  print'(1A15,1F9.5)','Err[lnDa](%) =',err_lnda*1d2
  print'(1A15,1F9.5)','Err[lnH](%)  =',err_lnh*1d2
  print'(1A15,1F9.5)','r(lnDa,lnH)  =',r12
  print'(1A15,1F9.5)','Err[lnR](%)  =',err_lnR*1d2
  DEALLOCATE(cov,work)
  return
END SUBROUTINE report_result
SUBROUTINE report_result3x3(fis)
  IMPLICIT none
  double precision, intent(IN) :: fis(3,3)
  double precision :: work(3),cov(3,3)
  print*,'*** Error on w, ignoring Omega_k and Omega_m ***'
  print'(1A20,1F9.5)','Err[w]            =',1d0/dsqrt(fis(1,1))
  cov=fis
  CALL DVERT(cov,3,3,work)
  print*,'*** Error on w, with Omega_k and Omega_m marginalized over ***'
  print'(1A20,1F9.5)','Err[w]            =',dsqrt(cov(1,1))
  print'(1A20,1F9.5)','Err[Omega_k]      =',dsqrt(cov(2,2))
  print'(1A20,1F9.5)','Err[lnOmega_m](%) =',dsqrt(cov(3,3))*1d2
  return
END SUBROUTINE report_result3x3
SUBROUTINE transform_fisher(z,fisDH,fis3x3)
  USE cosmo
  ! transform 2x2 Fisher matrix for (Da,H) to 3x3 matrix for 
  ! (w,Omega_k,ln(Omega_m)), and accumulate it [e.g., Eq.(30) of Shoji et al.]
  IMPLICIT none
  integer :: a,b,i,j
  double precision, intent(IN)    :: z,fisDH(2,2)
  double precision, intent(INOUT) :: fis3x3(3,3)
  double precision :: dpdq(3,2)
  double precision :: chi,h2,func0,func1,func2,func3,rombint
  external h2,func0,func1,func2,func3,rombint
  chi=rombint(func0,0d0,z,1d-7)
  dpdq(1,1)=-1.5d0*ode0*rombint(func1,0d0,z,1d-7)/chi         ! dlnDa/dw
  dpdq(1,2)= 1.5d0*ode0*dlog(1d0+z)/h2(z)                     ! dlnH/dw
  dpdq(2,1)=-0.5d0*rombint(func2,0d0,z,1d-7)/chi+chi**2d0/6d0 ! dlnDa/dOmega_k
  dpdq(2,2)= 0.5d0*(1d0+z)**2d0/h2(z)                         ! dlnH/dOmega_k
  dpdq(3,1)=-0.5d0*om0*rombint(func3,0d0,z,1d-7)/chi          ! dlnDa/dln(Omega_m)
  dpdq(3,2)= 0.5d0*om0*(1d0+z)**3d0/h2(z)                     ! dlnH/dln(Omega_m)
  do a=1,3
     do b=1,3
        do i=1,2
           do j=1,2
              ! transform and accumulate fis3x3
              fis3x3(a,b)=fis3x3(a,b)+dpdq(a,i)*dpdq(b,j)*fisDH(i,j)
           enddo
        enddo
     enddo
  enddo
  return
END SUBROUTINE transform_fisher
SUBROUTINE add_cmb(fis3x3)
  USE cosmo
  ! Add the distance information from CMB [e.g., Eq.(38) of Shoji et al.]
  IMPLICIT none
  integer :: a,b,i,j
  double precision :: da_accuracy=0.2d0 ! Percent error in Da(zcmb) 
  double precision :: om_accuracy=2d0   ! Percent error in Omega_m 
  double precision, intent(INOUT) :: fis3x3(3,3)
  double precision :: dpdq(3),zcmb=1090d0
  double precision :: chi,h2,func0,func1,func2,func3,rombint
  external h2,func0,func1,func2,func3,rombint
  chi=rombint(func0,0d0,zcmb,1d-7)
  dpdq(1)=-1.5d0*ode0*rombint(func1,0d0,zcmb,1d-7)/chi         ! dlnDa/dw
  dpdq(2)=-0.5d0*rombint(func2,0d0,zcmb,1d-7)/chi+chi**2d0/6d0 ! dlnDa/dOmega_k
  dpdq(3)=-0.5d0*om0*rombint(func3,0d0,zcmb,1d-7)/chi          ! dlnDa/dlnOmega_m
   do a=1,3
     do b=1,3
        ! add Da(z=1090)
        fis3x3(a,b)=fis3x3(a,b)+dpdq(a)*dpdq(b)*1d4/da_accuracy**2d0
     enddo
  enddo
  ! add Omega_matter
  fis3x3(3,3)=fis3x3(3,3)+1d4/om_accuracy**2d0
  print*,'<< CMB Priors Added (Edit da_accuracy & om_accuracy in "add_cmb") >>'
  print'(1A23,1F9.5)','Err[lnDa(z=1090)](%) =',da_accuracy
  print'(1A23,1F9.5)','Err[lnOmega_m](%)    =',om_accuracy
  return
END SUBROUTINE add_cmb
DOUBLE PRECISION FUNCTION h2(redshift)
  USE cosmo
  ! h2(z) = Omega_matter(1+z)^3+Omega_lambda
  IMPLICIT none
  DOUBLE PRECISION, intent(IN) :: redshift
  h2 = om0*(1d0+redshift)**3d0+ode0
  return
END FUNCTION h2
DOUBLE PRECISION FUNCTION func0(redshift)
  USE cosmo
  ! func0(z) = 1/[h2(z)]^0.5
  IMPLICIT none
  DOUBLE PRECISION, intent(IN) :: redshift
  DOUBLE PRECISION :: h2
  external :: h2
  func0 = 1d0/dsqrt(h2(redshift))
  return
END FUNCTION func0
DOUBLE PRECISION FUNCTION func1(redshift)
  USE cosmo
  ! func1(z) = ln(1+z)/[h2(z)]^1.5
  IMPLICIT none
  DOUBLE PRECISION, intent(IN) :: redshift
  DOUBLE PRECISION :: h2
  external :: h2
  func1 = dlog(1d0+redshift)/h2(redshift)**1.5d0
  return
END FUNCTION func1
DOUBLE PRECISION FUNCTION func2(redshift)
  USE cosmo
  ! func2(z) = (1+z)^2/[h2(z)]^1.5
  IMPLICIT none
  DOUBLE PRECISION, intent(IN) :: redshift
  DOUBLE PRECISION :: h2
  external :: h2
  func2 = (1d0+redshift)**2d0/h2(redshift)**1.5d0
  return
END FUNCTION func2
DOUBLE PRECISION FUNCTION func3(redshift)
  USE cosmo
  ! func3(z) = (1+z)^3/[h2(z)]^1.5
  IMPLICIT none
  DOUBLE PRECISION, intent(IN) :: redshift
  DOUBLE PRECISION :: h2
  external :: h2
  func3 = (1d0+redshift)**3d0/h2(redshift)**1.5d0
  return
END FUNCTION func3
